@@ -18,7 +18,7 @@ local operators
 local _function_factories = {}
 
 
-local utils = { _VERSION = "1.9.2" }
+local utils = { _VERSION = "1.10.0" }
 for k, v in pairs(compat) do utils[k] = v  end
 
 --- Some standard patterns
@@ -464,11 +464,12 @@ end
 
 --- split a string into a list of strings separated by a delimiter.
 -- @param s The input string
--- @param re A Lua string pattern; defaults to '%s+'
--- @param plain don't use Lua patterns
--- @param n optional maximum number of splits
+-- @param re optional A Lua string pattern; defaults to '%s+'
+-- @param plain optional If truthy don't use Lua patterns
+-- @param n optional maximum number of elements (if there are more, the last will remian un-split)
 -- @return a list-like table
 -- @raise error if s is not a string
+-- @see splitv
 function utils.split(s,re,plain,n)
     utils.assert_string(1,s)
     local find,sub,append = string.find, string.sub, table.insert
@@ -496,13 +497,19 @@ function utils.split(s,re,plain,n)
 end
 
 --- split a string into a number of return values.
+-- Identical to `split` but returns multiple sub-strings instead of
+-- a single list of sub-strings.
 -- @param s the string
--- @param re the delimiter, default space
+-- @param re A Lua string pattern; defaults to '%s+'
+-- @param plain don't use Lua patterns
+-- @param n optional maximum number of splits
 -- @return n values
--- @usage first,next = splitv('jane:doe',':')
+-- @usage first,next = splitv('user=jane=doe','=', false, 2)
+-- assert(first == "user")
+-- assert(next == "jane=doe")
 -- @see split
-function utils.splitv (s,re)
-    return _unpack(utils.split(s,re))
+function utils.splitv (s,re, plain, n)
+    return _unpack(utils.split(s,re, plain, n))
 end
 
 
@@ -557,6 +564,7 @@ local function _string_lambda(f)
     end
 end
 
+
 --- an anonymous function as a string. This string is either of the form
 -- '|args| expression' or is a function of one argument, '_'
 -- @param lf function as a string
@@ -587,6 +595,7 @@ function utils.bind1 (fn,p)
     return function(...) return fn(p,...) end
 end
 
+
 --- bind the second argument of the function to a value.
 -- @param fn a function of at least two values (may be an operator string)
 -- @param p a value
@@ -605,6 +614,102 @@ function utils.bind2 (fn,p)
     return function(x,...) return fn(x,p,...) end
 end
 
+
+
+
+--- Deprecation
+-- @section deprecation
+
+do
+  -- the default implementation
+  local deprecation_func = function(msg, trace)
+    warn(msg, "\n", trace)  -- luacheck: ignore
+  end
+
+  --- Sets a deprecation warning function.
+  -- An application can override this function to support proper output of
+  -- deprecation warnings. The warnings can be generated from libraries or
+  -- functions by calling `utils.raise_deprecation`. The default function
+  -- will write to the 'warn' system (introduced in Lua 5.4, or the compatibility
+  -- function from the `compat` module for earlier versions).
+  --
+  -- Note: only applications should set/change this function, libraries should not.
+  -- @param func a callback with signature: `function(msg, trace)` both arguments are strings.
+  -- @see utils.raise_deprecation
+  -- @usage
+  -- -- write to the Nginx logs with OpenResty
+  -- utils.set_deprecation_func(function(msg, trace)
+  --   ngx.log(ngx.WARN, msg, " ", trace)
+  -- end)
+  --
+  -- -- disable deprecation warnings
+  -- utils.set_deprecation_func()
+  function utils.set_deprecation_func(func)
+    if func == nil then
+      deprecation_func = function() end
+    else
+      utils.assert_arg(1, func, "function")
+      deprecation_func = func
+    end
+  end
+
+  --- raises a deprecation warning.
+  -- For options see the usage example below.
+  --
+  -- Note: the `opts.deprecated_after` field is the last version in which
+  -- a feature or option was NOT YET deprecated! Because when writing the code it
+  -- is quite often not known in what version the code will land. But the last
+  -- released version is usually known.
+  -- @param opts options table
+  -- @see utils.set_deprecation_func
+  -- @usage
+  -- warn("@on")   -- enable Lua warnings, they are usually off by default
+  --
+  -- function stringx.islower(str)
+  --   raise_deprecation {
+  --     source = "Penlight " .. utils._VERSION,                  -- optional
+  --     message = "function 'islower' was renamed to 'is_lower'" -- required
+  --     version_removed = "2.0.0",                               -- optional
+  --     deprecated_after = "1.2.3",                              -- optional
+  --   }
+  --   return stringx.is_lower(str)
+  -- end
+  -- -- output: "[Penlight 1.9.2] function 'islower' was renamed to 'is_lower' (deprecated after 1.2.3, scheduled for removal in 2.0.0)"
+  function utils.raise_deprecation(opts)
+    utils.assert_arg(1, opts, "table")
+    if type(opts.message) ~= "string" then
+      error("field 'message' of the options table must be a string", 2)
+    end
+    local trace = debug.traceback("", 2):match("[\n%s]*(.-)$")
+    local msg
+    if opts.deprecated_after and opts.version_removed then
+      msg = (" (deprecated after %s, scheduled for removal in %s)"):format(
+        tostring(opts.deprecated_after), tostring(opts.version_removed))
+    elseif opts.deprecated_after then
+      msg = (" (deprecated after %s)"):format(tostring(opts.deprecated_after))
+    elseif opts.version_removed then
+      msg = (" (scheduled for removal in %s)"):format(tostring(opts.version_removed))
+    else
+      msg = ""
+    end
+
+    msg = opts.message .. msg
+
+    if opts.source then
+      msg = "[" .. opts.source .."] " .. msg
+    else
+      if msg:sub(1,1) == "@" then
+        -- in Lua 5.4 "@" prefixed messages are control messages to the warn system
+        error("message cannot start with '@'", 2)
+      end
+    end
+
+    deprecation_func(msg, trace)
+  end
+
+end
+
+
 return utils
 
 
